@@ -20,17 +20,22 @@ database_engines = {}
 
 
 def get_table_key(table, obj=False):
-    """
-    This function takes an SQLAlchemy table object and returns its
+    """This function takes an SQLAlchemy table object and returns its
     primary key, or if the table has no primary key, it returns its
     first foreign key. Otherwise, it raises a ValidationError
 
-    @param table: An SQLAlchemy table object
-    @param obj: boolean, of whether the function is expected to return
-     an SQLAlchemy Column object. Otherwise, it returns a string, the
-     column name.
-    @return: Returns either the SQLAlchemy Column object or the
-    column name of the key column of the provided table.
+    Args:
+        table (SQLAlchemy.Table): An SQLAlchemy table object
+        obj (bool, optional): Whether the function is expected to return
+            an SQLAlchemy Column object. Otherwise, it returns a string, the
+            column name. Defaults to False.
+
+    Raises:
+        ValidationError: When the table has no key columns.
+
+    Returns:
+        (SQLAlchemy.Column|str): Returns either the SQLAlchemy Column object
+        or the column name of the key column of the provided table.
     """
     primary_key_columns = table.primary_key.columns.values()
     if len(primary_key_columns):
@@ -41,7 +46,7 @@ def get_table_key(table, obj=False):
             key = foreign_keys[0].column
         else:
             raise ValidationError(
-                'Table {0} has primary key or even foreign key'
+                'Table {0} has no primary key or even foreign key'
                 .format(table)
             )
     if obj:
@@ -50,14 +55,13 @@ def get_table_key(table, obj=False):
 
 
 def add_column(engine, table_name, column):
-    """
-    This function is used to add a new field to an already
+    """This function is used to add a new field to an already
     existing table in a database.
 
-    @param engine SQLAlchemy database engine object
-    @param table_name str, full path.name of the target table
-    @param column SQLAlchemy Column object that will be added
-    @returns No return value
+    Args:
+        engine (SQLAlchemy.Engine): Database engine object
+        table_name (str): full dottedpath.name of the target table
+        column (SQLAlchemy.Column): Column object that will be added
     """
     column_name = column.compile(dialect=engine.dialect)
     column_type = column.type.compile(engine.dialect)
@@ -71,12 +75,14 @@ def add_column(engine, table_name, column):
 
 
 def typeFullName(o):
-    """
-    This function takes any user defined object or class of any type and
+    """This function takes any user defined object or class of any type and
     returns a string absolute identifier of the provided type.
 
-    @param o: object of any type to be inspected
-    @returns: str The full path of the provided type
+    Args:
+        o (Any): object of any type to be inspected
+
+    Returns:
+        str: The full dotted path of the provided type
     """
     module = o.__class__.__module__
     if module is None or module == str.__class__.__module__:
@@ -106,14 +112,16 @@ def ColTypeToStr(Type):
 
 
 def StrToColType(TypePath):
-    """
-    This function takes a string SQLAlchemy Column type identifier
+    """This function takes a string SQLAlchemy Column type identifier
     and returns an SQLAlchemy type definition class of the provided type.
 
     This function is the inverse of the function ColTypeToStr.
 
-    @param TypePath: str, The string identifier of an SQLAlchemy column type.
-    @returns: class, the SQLAlchemy column class of the provided string.
+    Args:
+        TypePath (str): The string identifier of an SQLAlchemy column type.
+
+    Returns:
+        class: the SQLAlchemy column class of the provided string
     """
     import sqlalchemy
     pathentries = TypePath.split('.')
@@ -161,7 +169,7 @@ class Database(models.Model):
         This function returns an instance of an SQLAlchemy database_engine of
          the database configured in the object calling the method.
 
-        For perfomance, each time this function is called, it saved a copy of
+        For perfomance, each time this function is called, it saves a copy of
          the engine in the database_engines dictionary so that when it's called
          again for the same database, it will first look in the dictionary and
          if it find that the engine is already saved there, it will return it
@@ -203,14 +211,14 @@ class Database(models.Model):
             results = []
             if self.source == datasources.SQLIGHT:
                 ret = connection.execute(make_query(datasources.__list__[
-                                    self.source]['dialect'] + '/list_tables'))
+                    self.source]['dialect'] + '/list_tables'))
             else:
                 ret = connection.execute(make_query('list_tables'))
             for record in ret:
                 results.append(record[0])
             return results
 
-    def meta(self, schema=None):
+    def meta(self, schema=None, refresh=False):
         """
         This function returns an SQLAlchemy database MetaData object of the
          database from which the function is being called.
@@ -220,10 +228,18 @@ class Database(models.Model):
          that when it's called again for the same database, it will first
          look in the dictionary and if it found that the MetaDate is already
          saved there, it will return it directly from there.
+
+        Parameters
+        ----------
+            schema (str | None): The schema name to get the MetaData for.
+                Default is 'public'
+
+            refresh (boolean): Whether to reload the MetaData from the
+                server or use the cached copy instead for performance.
         """
         db = self.mount()
         metaid = '{0}.{1}'.format(self.pk, schema)
-        if metaid in cached_database_metas:
+        if metaid in cached_database_metas and not refresh:
             return cached_database_metas[metaid]
         else:
             logging.debug(
@@ -234,18 +250,23 @@ class Database(models.Model):
             cached_database_metas[metaid].reflect(bind=db)
             return cached_database_metas[metaid]
 
-    def get_table(self, table, schema=None):
+    def get_table(self, table, schema=None, refresh=False):
         """
         This function returns an SQLAlchemy Table object for the table
          identified by 'table', and 'schema' (optional)
+
+        Parameters
+        ----------
+            table (str): The name of the table to retrive
+
+            schema (str | None): The schema containing the intended table.
+                Default is None, which refers to the public schema.
+
+            refresh (boolean): Whether to load the table structure from the
+                database or to use the cached meta data instead
+                for performance.
         """
-        meta = self.meta(schema)
-        if(len(meta.tables.keys()) < 1):
-            raise Exception(
-                'Exception in function Database.get_table. '
-                '{0}.meta() returned an empty structure'
-                .format(self)
-            )
+        meta = self.meta(schema, refresh=refresh)
         if schema is not None:
             table = schema + '.' + table
         if table in meta.tables:
@@ -255,7 +276,7 @@ class Database(models.Model):
 
     def clean(self):
         """
-        This function is used to calidate database connection information
+        This function is used to validate database connection information
          before saving it.
         """
         try:
@@ -275,19 +296,39 @@ class SourceTable(models.Model):
      can be added as a SourceTable. Once added, it's being monitored by the
      PDR server; so that whenever any changes occur in any record in that
      table, the PDR server will be notifed in order to update reflections.
+
+    When a talbe is added as a source table, trigger function are installed
+    inside the database intself, so that whenever any INSERT, UPDATE, or DELETE
+    events occure on any record in that table, the functions insert a record to
+    a pdr_event table inside the same database which is being used to record
+    the history of changes of that table. Each record in the pdr_event table
+    records what event has occured, (INSERT UPDATE OR DELETE) the timestamp
+    at which the event has occured, and the record affected is being identified
+    by the value of its primary key column. That's why source tables must have
+    a primary key column for a reliable way to refer to each individual record.
     """
     source_database = models.ForeignKey(Database, on_delete=models.CASCADE)
     source_table = models.CharField(max_length=200)
     description = models.CharField(max_length=500, blank=True)
 
-    def get_table(self):
+    def get_table(self, refresh=False):
         """
         Retrives the SQLAlchemy Table object of the source table from which
          the function was called.
+
+        Parameters
+        ----------
+
+            refresh (boolean): Whether to load the table structure from the
+                database or to use cached meta data instead for performance.
+
+        Returns
+        -------
+            Class(SQLAlchemy.Table)
         """
         path = self.source_table.split('.')
         path.reverse()
-        ret = self.source_database.get_table(*path)
+        ret = self.source_database.get_table(*path, refresh=refresh)
         if type(ret).__name__ != 'Table':
             raise Exception(
                 'Exception in function SourceTabl.get_table. '
@@ -299,24 +340,44 @@ class SourceTable(models.Model):
             )
         return ret
 
-    def get_pdr_table(self):
+    def get_pdr_table(self, refresh=False):
         """
         Retrives the notification channel Table object (AKA pdr_table) of the
          source table from which the function was called.
+
+        Parameters
+        ----------
+
+            refresh (boolean): Whether to load the table structure from the
+                database or to use cached meta data instead for performance.
+
+        Returns
+        -------
+            Class(SQLAlchemy.Table)
         """
         table_path = self.source_table.split('.')
         if len(table_path) == 1:
             table_path.insert(0, 'None')
         pdr_table_name = '{0}_o_{1}_o_{2}'.format(pdr_prefix, *table_path)
-        return self.source_database.get_table(pdr_table_name)
+        return self.source_database.get_table(pdr_table_name, refresh=refresh)
 
-    def get_structure(self):
+    def get_structure(self, refresh=False):
         """
         Retrives a dict object describing the structure of the source table
          from which the function was called.
+
+        Parameters
+        ----------
+
+            refresh (boolean): Whether to load the table structure from the
+                database or to use cached meta data instead for performance.
+
+        Returns
+        -------
+            dict:
         """
         ret = {"columns": {}}
-        table = self.get_table()
+        table = self.get_table(refresh=refresh)
         if type(table).__name__ != 'Table':
             raise Exception(
                 'Exceeption in function SourceTable. get_structure.'
@@ -440,6 +501,75 @@ class SourceTable(models.Model):
 
 
 class Reflection(models.Model):
+    """A Reflection is a group of configurations and instructions that
+    describe a replication of a table. Minimally, a reflection describes
+    the destination database and the destination table to which data will
+    be replicated and the source table from which data will be retrived.
+    And by default, the destination table will have the same columes as
+    the source table from which it's being replicated. However, PDR provdes
+    more advanced option to customize the structure of the destination table
+    which keeping it up-to-date with its source table.
+
+    A reflection is described by the following three strings:
+        source_fields:
+            This field describes the structure of the source table (a list of
+            column names and data types and defines which column is the
+            primarykey)
+            source_fields are auto generated from the source database and
+            cannot be modified
+
+        destination_table:
+            The name of the destination table. (it will be created if it
+            didn't exist)
+
+        destination_fields:
+            This field descibes the structure of the destination table and it's
+            being used to create the table if it did not exist or to add
+            columns to it if it already existed but didn't contain all the
+            columns listed in the destination_table
+
+            Note: The table can contain more columns than that listed in
+                destination_fields. However, it must have the same primary
+                key as configured in the destination_fields variable.
+
+            key_binding: In addition to the columns, and the primary key,
+                there's a key_binding optional variable defined in the
+                destination_fields object. The syntax for this variable is:
+                { source_key : destination_key }
+                By default, source_key will be the name of the primary key
+                column name of the source table, and destination_key will be
+                the name of the primary key column name of the destination
+                table. However, if the column intended to be the primary key
+                for the destination table is not the same as that of the
+                source table, we might have some trouble applying the
+                replication because PDR needs to be able to identify each
+                idividual record in the destination table and match it with
+                the record it came from in the source table. That's why
+                key_binding must be used when the source and destination
+                talbes has different primary key columns.
+
+                Conditions:
+                ----------
+                The following conditions must be met when using key_binding
+
+                    1. Both of the source_key and the destination_key must be
+                        of the same data type.
+                    2. destination_key must be the only required field in the
+                        destination table because during replication, the
+                        PDR server while chech which records exist and which
+                        doesn't based on the value of that column and will
+                        create non-existing records by inserting nothing but
+                        the destination_key field for these records.
+                        Tl;Dr: destination_key must be the primary key of the
+                        destination table.
+
+        reflection_statment:
+            reflection_statment is an SQL update statment applied on every
+        record in the destination table to reflect the changes on that table.
+        The reflection statment is manipulated using SQLALchemy's bindparams
+        function with the values of each column in source_fields before
+        sending the query to the server.
+    """
     description = models.CharField(max_length=500)
     source_table = models.ForeignKey(  # The source table
         SourceTable,
@@ -494,15 +624,26 @@ class Reflection(models.Model):
         max_length=10000
     )
 
-    def get_destination_table(self):
+    def get_destination_table(self, refresh=False):
+        """
+        Parameters
+        ----------
+
+            refresh (boolean): Whether to load the table structure from the
+                database or to use cached meta data instead for performance.
+
+        Returns
+        -------
+            Class(SQLAlchemy.Table)
+        """
         dest_table_path = self.destination_table.split('.')
         dest_table_path.reverse()
         destination_table = self.destination_database.get_table(
-            *dest_table_path)
+            *dest_table_path, refresh=refresh)
         return destination_table
 
-    def get_source_table(self):
-        return self.source_table.get_table()
+    def get_source_table(self, refresh=False):
+        return self.source_table.get_table(refresh=refresh)
 
     def get_destination_structure(self):
         return json.loads(self.destination_fields)
@@ -800,7 +941,11 @@ class Reflection(models.Model):
             raise ValidationError(
                 'Unable to parse json: {0}'.format(self.destination_fields))
         ddb = self.destination_database.mount()
-        destinationTable = self.destination_database.get_table(table, schema)
+        destinationTable = self.destination_database.get_table(
+            table,
+            schema,
+            refresh=True
+        )
         if destinationTable is not None:
             # Table already exists, check its compatiblity
             logging.info(
